const paths = require('./paths');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const ProgressPlugin = require('progress-bar-webpack-plugin');
const chalk = require('chalk');
const Webpackbar = require('webpackbar');
const MiniCssExtractPlugin = require('mini-css-extract-plugin');
const ErrorOverlayPlugin = require('error-overlay-webpack-plugin')

// style config
const cssReg = /\.css$/;
const cssModuleReg = /\.module\.css$/;
const sassReg = /\.s(a|c)ss$/;
const sassModuleReg = /\.module\.s(a|c)ss$/;
const lessReg = /\.less$/;
const lessModuleReg = /\.module\.less$/;

module.exports = function(mode) {
  const isDev = mode === 'development' ? true : false;
  const isProd = mode === 'production' ? true : false;

  const getStyleLoaders = (cssOptions, preProcessor) => {
    const loaders = [
      isDev && require.resolve('style-loader'),
      isProd && {
        loader: MiniCssExtractPlugin.loader
      },
      {
        loader: require.resolve('css-loader'),
        options: cssOptions
      },
      {
        loader: require.resolve('postcss-loader'),
        options: {
          plugins: [require('autoprefixer')]
        }
      }
    ].filter(Boolean);

    // console.log(loaders);

    if (preProcessor) {
      loaders.push({
        loader: require.resolve(preProcessor),
        options: {
          sourceMap: true
        }
      });
    }

    return loaders;
  };

  return {
    // development or production mode
    mode,

    // devtool
    devtool: isDev ? 'cheap-module-source-map' : isProd && 'source-map',

    // app entry
    entry: [
      // isDev && require.resolve('./tools/webpackHotDevClient.js'),
      paths.entry
    ].filter(Boolean),

    // app build output
    output: {
      path: paths.build,
      filename: 'static/js/bundle.js',
      chunkFilename: isProd
        ? 'static/js/[name].[contenthash:8].js'
        : isDev && 'static/js/[name].chunk.js',
      publicPath: '/'
    },

    // optimize
    optimization: {
      splitChunks: {
        name: true,
        chunks: 'all'
      },
      runtimeChunk: true
    },

    // module config
    module: {
      rules: [
        {
          oneOf: [
            // url-loader
            {
              test: [/\.bmp$/, /\.gif$/, /\.jpe?g$/, /\.png$/],
              loader: require.resolve('url-loader'),
              options: {
                limit: '10000',
                name: 'static/media/[name].[hash:8].[ext]'
              }
            },

            // babel-loader
            {
              test: /(js|ts)x?/,
              use: {
                loader: require.resolve('babel-loader'),
                options: {
                  compact: false,
                  presets: [require.resolve('babel-preset-react-app')]
                }
              }
            },

            // style loader
            {
              test: cssReg,
              exclude: cssModuleReg,
              use: getStyleLoaders({
                importLoaders: 1,
                sourceMap: isProd
              })
            },
            {
              test: cssModuleReg,
              use: getStyleLoaders({
                importLoaders: 1,
                modules: true,
                sourceMap: isProd
              })
            },

            // sass loader
            {
              test: sassReg,
              exclude: sassModuleReg,
              use: getStyleLoaders(
                {
                  importLoaders: 2,
                  sourceMap: isProd
                },
                'sass-loader'
              ),
              sideEffects: true
            },
            {
              test: sassModuleReg,
              use: getStyleLoaders(
                {
                  importLoaders: 2,
                  sourceMap: isProd,
                  modules: true
                },
                'sass-loader'
              ),
              sideEffects: true
            },

            // less loader
            {
              test: lessReg,
              exclude: lessModuleReg,
              use: getStyleLoaders(
                {
                  importLoaders: 2,
                  sourceMap: isProd
                },
                'less-loader'
              )
            },
            {
              test: lessModuleReg,
              use: getStyleLoaders(
                {
                  importLoaders: 2,
                  sourceMap: isProd,
                  modules: true
                },
                'less-loader'
              )
            },

            // file-loader media
            {
              loader: require.resolve('file-loader'),
              exclude: [/\.(js|mjs|jsx|ts|tsx)$/, /\.html$/, /\.json$/],
              options: {
                name: 'static/media/[name].[hash:8].[ext]'
              }
            }
          ]
        }
      ]
    },

    // plugins
    plugins: [
      new HtmlWebpackPlugin({
        inject: true,
        template: paths.publicHTML
      }),

      // progress
      process.platform === 'win32'
        ? new ProgressPlugin({
            format:
              '  build [:bar] ' +
              chalk.green.bold(':percent') +
              ' (:elapsed seconds)',
            clear: false
          })
        : new Webpackbar({
            name: isProd ? 'build' : 'server',
            color: 'orange',
            profile: isProd ? true : false
          }),

      // mini extract css
      isProd &&
        new MiniCssExtractPlugin({
          filename: 'static/css/[name].[contenthash:8].css',
          chunkFilename: 'static/css/[name].[contenthash:8].chunk.css'
        }),

      // overlay
      isDev && new ErrorOverlayPlugin(),
    ].filter(Boolean),

    // resolve: do not forget the . point
    resolve: {
      extensions: ['.js', '.jsx', '.ts', '.tsx']
    },

    performance: false
  };
};
