const webpackConfig = require("../config/webpack.config.js");
const webpack = require("webpack");
const chalk = require("chalk");
const formatWebpackMessages = require("../config/tools/formatWebpackMessages");
const { checkBrowsers } = require("../config/tools/browserHelper");
const paths = require("../config/paths");
const fs = require("fs-extra");
const FileSizeReporter = require("../config/tools/FileSizeReporter");

process.env.NODE_ENV = "production";
process.env.BABEL_ENVy = "production";

process.on("unhandledRejection", err => {
  throw err;
});

// build
const config = webpackConfig("production");
const compiler = webpack(config);

const WARN_AFTER_BUNDLE_GZIP_SIZE = 512 * 1024;
const WARN_AFTER_CHUNK_GZIP_SIZE = 1024 * 1024;
const measureFileSizesBeforeBuild =
  FileSizeReporter.measureFileSizesBeforeBuild;
const printFileSizesAfterBuild = FileSizeReporter.printFileSizesAfterBuild;

checkBrowsers(paths.app)
  .then(() => {
    return measureFileSizesBeforeBuild(paths.build);
  })
  .then(previousFileSizes => {
    fs.emptyDirSync(paths.build);
    copyPublicFolders();
    return build(previousFileSizes);
  })
  .then(({ stats, previousFileSizes, warnings }) => {
    if (warnings.length) {
      console.log(chalk.yellow("Compiled with warnings.\n"));
      console.log(warnings.join("\n\n"));
    } else {
      console.log(chalk.bgGreenBright("Compiled successfully...\n"));
    }

    console.log("File sizes after gzip:\n");
    printFileSizesAfterBuild(
      stats,
      previousFileSizes,
      paths.build,
      WARN_AFTER_BUNDLE_GZIP_SIZE,
      WARN_AFTER_CHUNK_GZIP_SIZE
    );
    console.log();
  }, err => {
    console.log(chalk.red('Failed to compile.\n'));
    process.exit(1);
  });

function build(previousFileSizes) {
  console.log(
    chalk.bgYellow(chalk.blue("Creating an optimized production build..."))
  );
  return new Promise((resolve, reject) => {
    return compiler.run((err, stats) => {
      var messages;
      if (err) {
        if (!err.message) {
          return reject(err);
        }
        messages = {
          errors: [err.message],
          warnings: []
        };
      } else {
        messages = formatWebpackMessages(
          stats.toJson({ all: false, warnings: true, errors: true })
        );
      }

      if (messages.errors.length) {
        if (messages.errors.length > 1) {
          messages.errors.length = 1;
        }
        return reject(new Error(messages.errors.join("\n\n")));
      }

      resolve({
        previousFileSizes,
        stats,
        warnings: messages.warnings
      });
    });
  });
}

function copyPublicFolders() {
  fs.copySync(paths.public, paths.build, {
    dereference: true,
    filter: file => file !== paths.publicHTML
  });
}
