const WebpackDevServer = require('webpack-dev-server');
const webpack = require('webpack');
const webpackConfig = require('../config/webpack.config.js');
const paths = require('../config/paths');
const fs = require('fs-extra');
const chalk = require('chalk');
const { checkBrowsers } = require('../config/tools/browserHelper');
const choosePort = require('../config/tools/choosePort');
const prepareUrls = require('../config/tools/prepareUrls');
const openBrowser = require('../config/tools/openBrowser');
const formatWebpackMessages = require('../config/tools/formatWebpackMessages');

process.env.NODE_ENV = 'development';

const NOMOCK = process.env.MOCK === 'none' ? true : false;
const HOST = process.env.HOST || '0.0.0.0';
const PORT = process.env.PORT || 8888;

checkBrowsers(paths.app)
  .then(() => {
    return choosePort(HOST, PORT);
  })
  .then(port => {
    if (!port) {
      return;
    }
    // console.log(port);
    const config = webpackConfig('development');
    const appName = require(paths.package).name;
    const protocol = process.env.HTTP ? 'https' : 'http';
    const urls = prepareUrls(protocol, HOST, PORT);

    const compiler = webpack(config);
    var firstCompile = true;
    // compiler hooks
    compiler.hooks.beforeCompile.tap('beforeCompile', () => {
      // console.log("before compile");
    });

    compiler.hooks.done.tap('done', stats => {
      const statsData = stats.toJson({
        all: false,
        errors: true,
        warnings: true
      });
      // console.log(statsData);
      const { errors, warnings } = statsData;
      var isSuccessful = !errors.length && !warnings.length;

      if (isSuccessful) {
        if (firstCompile) {
          console.log();
          console.log(
            chalk.greenBright(
              `You can now view ${chalk.bold(appName)} in the browser.`
            )
          );
          console.log();
          if (urls.lanUrlForTerminal) {
            console.log(
              `  ${chalk.bold('Local:')}            ${chalk.yellowBright(
                urls.localUrlForTerminal
              )}`
            );
            console.log(
              `  ${chalk.bold('On Your Network:')}  ${chalk.yellowBright(
                urls.lanUrlForTerminal
              )}`
            );
          } else {
            console.log(`  ${chalk.yellowBright(urls.localUrlForTerminal)}`);
          }
        }

        firstCompile = false;
        console.log(chalk.bgGreen('\n DONE '));
      }

      var messages = formatWebpackMessages(statsData);
      if (messages.errors.length) {
        console.log(chalk.red('Failed to compile.\n'));
        console.log(messages.errors.join('\n\n'));
        return;
      }

      if (messages.warnings.length) {
        console.log(chalk.yellow('Compiled with warnings.\n'));
        console.log(messages.warnings.join('\n\n'));
      }
    });

    compiler.hooks.invalid.tap('invalid', () => {
      console.log();
      console.log(chalk.bgMagenta(' WAIT ') + chalk.whiteBright(' Compile...'));
    });

    // handle proxy && devServer
    const proxy = require(paths.proxy).proxy;
    const devServerConfig = {
      contentBase: paths.content,
      quiet: true,
      overlay: false,
      publicPath: '/'
    };

    if (!NOMOCK) {
      const methods = ['get', 'post', 'put', 'delete', 'options'];
      if (!fs.existsSync(paths.mock)) {
        console.log();
        console.log(
          chalk.yellowBright.bold(
            'The app support mock data, if you want to mock some data, please make a mock directory in the root dir.\n'
          )
        );
        console.log(
          chalk.greenBright.bold('Introduce: mock directory example below \n\nmock/user.js\nmodule.exports = {\n  "GET /api/user": {} or [] or function(req, res) {}(express reference)\n}\n')
        );
      } else {
        devServerConfig.after = function(app) {
          fs.readdir(paths.mock, function(err, files) {
            if (err) {
              return console.log(err);
            }
            if(!files.length) {
              console.log(chalk.whiteBright.bold('The mock dir has exists!!!'))
              console.log(
                chalk.greenBright.bold('Introduce: mock directory example below \n\nmock/user.js\nmodule.exports = {\n  "GET /api/user": {} or [] or function(req, res) {}(express reference)\n}\n')
              );
            }
            files.map(file => {
              /* get module.exports object */
              let exp = require(`${paths.mock}/${file}`);
              for (let key in exp) {
                let splits = key.split(' ');
                let method = splits[0].toLowerCase(),
                  url = splits[1];

                if (methods.includes(method)) {
                  app[method](url, (req, res) => {
                    if (typeof exp[key] === 'function') {
                      return exp[key](req, res);
                    }
                    return res.status(200).send(exp[key]);
                  });
                } else {
                  console.log();
                  console.log(
                    chalk.red('The MOCK data maybe not support ' + method)
                  );
                  console.log();
                }
              }
            });
          });
        };
      }
    }

    if (NOMOCK && proxy) {
      devServerConfig.proxy = proxy;
    }

    const devServer = new WebpackDevServer(compiler, devServerConfig);

    devServer.listen(port, HOST, function(error) {
      if (error) {
        console.log(error);
        process.exit(1);
      }
      console.log(chalk.magentaBright('start the server development...'));
      openBrowser(urls.localLanUrlForBrowser);
    });

    ['SIGINT', 'SIGTERM'].forEach(function(sig) {
      process.on(sig, function() {
        devServer.close();
        process.exit();
      });
    });
  })
  .catch(err => {
    if (err && err.message) {
      console.log(err.message);
    }
    process.exit(1);
  });
